--[[

$Revision: 54933 $

(C) Copyright 2007 Bethink (bethink@naef.com)

This file is part of CrossRealm.

CrossRealm is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CrossRealm is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CrossRealm. If not, see <http://www.gnu.org/licenses/>.

]]


----------------------------------------------------------------------
-- Acquire libraries and declare add-on

local L = AceLibrary("AceLocale-2.2"):new("CrossRealm")
local Waterfall = AceLibrary("Waterfall-1.0")
CrossRealm = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0")


----------------------------------------------------------------------
-- Constants

-- Quality map
local qualityMap = {
	[0] = L["POOR"],
	[1] = L["COMMON"],
	[2] = L["UNCOMMON"],
	[3] = L["RARE"],
	[4] = L["EPIC"]
}
local inverseQualityMap = { }
for key, value in pairs(qualityMap) do
	inverseQualityMap[value] = key
end

-- Chat options
local opts = {
	type = "group",
	args = {
		minProfit = {
			type = "text",
			name = L["MIN_PROFIT"],
			desc = L["MIN_PROFIT_DESC"],
			usage = L["MIN_PROFIT_USAGE"],
			get = "GetMinProfit",
			set = "SetMinProfit",
			validate = "ValidateMinProfit"
		},
		minProfitPct = {
			type = "text",
			name = L["MIN_PROFIT_PCT"],
			desc = L["MIN_PROFIT_PCT_DESC"],
			usage = L["MIN_PROFIT_PCT_USAGE"],
			get = "GetMinProfitPct",
			set = "SetMinProfitPct",
			validate = "ValidateMinProfitPct"
		},
		minQuality = {
			type = "text",
			name = L["MIN_QUALITY"],
			desc = L["MIN_QUALITY_DESC"],
			get = "GetMinQuality",
			set = "SetMinQuality",
			validate = {
				L["POOR"],
				L["COMMON"],
				L["UNCOMMON"],
				L["RARE"],
				L["EPIC"]
			}
		},
		waterfall = {
			type = "execute",
			name = L["WATERFALL"],
			desc = L["WATERFALL_DESC"],
			func = "Waterfall"
		},
		wowecon = {
			type = "execute",
			name = L["WOWECON"],
			desc = L["WOWECON_DESC"],
			func = "ReportWowecon"
		}
	}
}


----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function CrossRealm:OnInitialize()
	-- Register chat commands
	self:RegisterChatCommand(L["SLASH_COMMANDS"], opts)
	
	-- Register configuration dialog
	opts.waterfall = nil
	Waterfall:Register("CrossRealm", "aceOptions", opts)
	
	-- Register database
	self:RegisterDB("CrossRealmDB", "CrossRealmDBPerCharacter")
	self:RegisterDefaults("profile", {
		minProfit = 10000, -- 1 gold
		minProfitPct = 20, -- 20 percent
		minQuality = 3 -- rare
	})
end


----------------------------------------------------------------------
-- Settings

-- Returns the minimum profit
function CrossRealm:GetMinProfit ()
	return self.db.profile.minProfit
end

-- Sets the minimum profit
function CrossRealm:SetMinProfit (minProfit)
	self.db.profile.minProfit = tonumber(minProfit)
end

-- Validates the minimum profit
function CrossRealm:ValidateMinProfit (minProfit)
	local number = tonumber(minProfit)
	return number ~= nil and number >= 0
end

-- Returns the minimum profit in percent
function CrossRealm:GetMinProfitPct ()
	return self.db.profile.minProfitPct
end

-- Sets the minimum profit in percent
function CrossRealm:SetMinProfitPct (minProfitPct)
	self.db.profile.minProfitPct = tonumber(minProfitPct)
end

-- Validates the minimum profit in percent
function CrossRealm:ValidateMinProfitPct (minProfitPct)
	local number = tonumber(minProfitPct)
	return number ~= nil and number >= 0 and number <= 100
end

-- Returns the minimum quality
function CrossRealm:GetMinQuality ()
	return qualityMap[self.db.profile.minQuality]
end

-- Sets the minimum quality
function CrossRealm:SetMinQuality (minQuality)
	self.db.profile.minQuality = inverseQualityMap[minQuality]
end


----------------------------------------------------------------------
-- Executes

-- Shows the configuration dialog
function CrossRealm:Waterfall ()
	Waterfall:Open("CrossRealm")
end


----------------------------------------------------------------------
-- Reporting

-- Generates a report based on the WOWEcon.com reference price source
function CrossRealm:ReportWowecon ()
	-- Check is WOWEcon_PriceMod is available
	if not IsAddOnLoaded("WOWEcon_PriceMod") then
		DEFAULT_CHAT_FRAME:AddMessage(L["WOWECON_PRICEMOD_NOT_LOADED"])
		return
	end

	-- Load auctioneer as needed
	if not IsAddOnLoaded("Auctioneer") then
		LoadAddOn("Auctioneer")
	end

	-- Get a snapshot and process
	local auctions = Auctioneer.SnapshotDB.Query(nil, nil, CrossRealm.AuctioneerFilter, nil)
	for _, auction in pairs(auctions) do
		local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
		local itemLink = Auctioneer.ItemDB.GetItemLink(itemKey)
		local woweconPrice = Wowecon.API.GetAuctionPrice_ByLink(itemLink, Wowecon.API.GLOBAL_PRICE)
		if woweconPrice then
			local buyoutPrice = auction.buyoutPrice / auction.count
			local profit = woweconPrice - buyoutPrice
			local profitPct = profit / buyoutPrice * 100
			if profit >= self.db.profile.minProfit and profitPct >= self.db.profile.minProfitPct then
				DEFAULT_CHAT_FRAME:AddMessage(format(
					L["REPORT_LINE"],
					itemLink,
					auction.count,
					buyoutPrice / 10000,
					woweconPrice / 10000,
					profit / 10000,
					profitPct))
			end
		end
	end
end

-- Filters auctions from Auctioneer
function CrossRealm.AuctioneerFilter (auction, filterArgs)
	-- Check quality
	local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
	local itemInfo = Auctioneer.ItemDB.GetItemInfo(itemKey)
	if itemInfo.quality < CrossRealm.db.profile.minQuality then
		return false
	end

	-- Check for buyout auction
	if auction.buyoutPrice == 0 then
		return false
	end

	-- Passed
	return true
end




